from uuid import uuid4

import pytest
from orchestrator.db import ProductTable, db
from pydantic_forms.exceptions import FormValidationError
from sqlalchemy import select

from gso.services.partners import filter_partners_by_name
from test.fixtures import create_subscription_for_mapping
from test.workflows import assert_complete, run_workflow

CORRECT_SUBSCRIPTION = str(uuid4())


def get_one_product(product_name):
    return db.session.scalars(select(ProductTable).where(ProductTable.name == product_name)).one()


@pytest.mark.workflow()
def test_delete_partner_success(partner_factory):
    partner = partner_factory(
        name="new_name",
        email="myemail@gmail.com",
    )

    assert filter_partners_by_name(name="new_name", case_sensitive=False)

    result, _, _ = run_workflow(
        "task_delete_partners",
        [
            {"partners": partner["partner_id"]},
        ],
    )
    assert_complete(result)

    assert filter_partners_by_name(name="new_name", case_sensitive=False) is None


def test_delete_partner_with_associated_data_fails(generic_product_3, partner_factory):
    partner = partner_factory(
        name="new_name",
        email="myemail@gmail.com",
    )

    subscription_mapping = {"PB_2": [{"rt_3": "info.id", "rt_2": "info2.id"}]}
    values = {"info.id": "0", "info2.id": "X"}
    product = get_one_product("Product 3")
    create_subscription_for_mapping(
        product, subscription_mapping, values, subscription_id=CORRECT_SUBSCRIPTION, customer_id=partner["partner_id"]
    )

    with pytest.raises(FormValidationError) as error:
        run_workflow(
            "task_delete_partners",
            [
                {"partners": partner["partner_id"]},
            ],
        )

    assert error.value.errors[0]["msg"] == "This partner has associated data and cannot be removed."
